#include <glibmm/refptr.h>

#include <cassert>
#include <iostream>
#include "MediatorSample.h"

using Glib::RefPtr;

AppWindow::AppWindow()
  : mAddButton(Gtk::manage(new AddButton(this)))
  , mDeleteButton(Gtk::manage(new DeleteButton(this)))
  , mListView(/*columns_count = */ 1)
{
  set_title("Mediator Sample");
  set_default_size(440, 480);
  move(100, 100);

  add(mFixedContainer);
  mFixedContainer.add(*mAddButton);
  mFixedContainer.add(*mDeleteButton);
  mFixedContainer.add(mInputField);
  mFixedContainer.add(mListView);

  mAddButton->set_label("Add");
  mAddButton->set_size_request(80, 20);
  mDeleteButton->set_label("Delete");
  mDeleteButton->set_size_request(80, 20);

  mListView.set_column_title(0, "Items");

  mInputField.set_size_request(200, 20);
  mListView.set_size_request(400, 400);

  mFixedContainer.move(mInputField, 20, 20);
  mFixedContainer.move(*mAddButton, 240, 20);
  mFixedContainer.move(*mDeleteButton, 340, 20);
  mFixedContainer.move(mListView, 20, 60);

  mFixedContainer.show_all();
}

void
AppWindow::ChangeState(Colleague* aEventSource)
{
  assert(aEventSource == mAddButton || aEventSource == mDeleteButton);

  if (aEventSource == mAddButton) {
    std::string text = mInputField.get_text();

    if (! text.empty()) {
      std::cout << "Adding " << text << "\n";

      mListView.append(text);

      mInputField.set_text("");
    }
  } else /* if (aEventSource == mDeleteButton)*/ {
    std::vector<int> selections = mListView.get_selected();
    assert(selections.size() <= 1);

    if (selections.size() > 0) {
        int selectedRow = selections[0];
        std::cout << "Deleting " << mListView.get_text(selectedRow) << std::endl;

        RefPtr<Gtk::ListStore> model =
          RefPtr<Gtk::ListStore>::cast_dynamic(mListView.get_model());

        if (model) {
          auto iterator = model->children()[selectedRow];
          model->erase(iterator);
        }
    }
  }
}

AddButton::AddButton(AppWindow* aMediator)
  : mMediator(aMediator)
{
}

void
AddButton::on_clicked()
{
  std::cout << "AddButton::on_clicked() is called.\n";

  // Call the default on_clicked() handler.
  Gtk::Button::on_clicked();

  mMediator->ChangeState(this);
}

DeleteButton::DeleteButton(AppWindow* aMediator)
  : mMediator(aMediator)
{
}

void
DeleteButton::on_clicked()
{
  std::cout << "DeleteButton::on_clicked() is called.\n";

  // Call the default on_clicked() handler.
  Gtk::Button::on_clicked();

  mMediator->ChangeState(this);
}

int main(int argc, char *argv[])
{
  auto app =
    Gtk::Application::create(argc, argv,
      "patterns.mvc");

  AppWindow window;

  return app->run(window);
}
